@@ -35,11 +35,11 @@ package Coro::SemaphoreSet;
 
 use common::sense;
 
-our $VERSION = 6.41;
+our $VERSION = 6.514;
 
 use Coro::Semaphore ();
 
-=item new [inital count]
+=item new [initial count]
 
 Creates a new semaphore set with the given initial lock count for each
 individual semaphore. See L<Coro::Semaphore>.
@@ -116,7 +116,11 @@ otherwise return false and leave the semaphore unchanged.
 =cut
 
 sub try {
-   Coro::Semaphore::try ($_[0][1]{$_[1]} || return $_[0][0] > 0)
+   Coro::Semaphore::try (
+      $_[0][1]{$_[1]} ||= $_[0][0] > 0
+         ? Coro::Semaphore::_alloc $_[0][0]
+         : return 0
+   )
 }
 
 =item $semset->count ($id)
@@ -190,10 +194,10 @@ sub sem {
 
 =back
 
-=head1 AUTHOR
+=head1 AUTHOR/SUPPORT/CONTACT
 
- Marc Lehmann <schmorp@schmorp.de>
- http://home.schmorp.de/
+   Marc A. Lehmann <schmorp@schmorp.de>
+   http://software.schmorp.de/pkg/Coro.html
 
 =cut
 
