@@ -36,12 +36,12 @@ package Coro::Specific;
 
 use common::sense;
 
-our $VERSION = 6.39;
+our $VERSION = 6.514;
 
 =item new
 
 Create a new coroutine-specific scalar and return a reference to it. The
-scalar is guarenteed to be "undef". Once such a scalar has been allocated
+scalar is guaranteed to be "undef". Once such a scalar has been allocated
 you cannot deallocate it (yet), so allocate only when you must.
 
 =cut
@@ -80,10 +80,10 @@ sub STORE {
 The actual coroutine specific values do not automatically get destroyed
 when the Coro::Specific object gets destroyed.
 
-=head1 AUTHOR
+=head1 AUTHOR/SUPPORT/CONTACT
 
- Marc Lehmann <schmorp@schmorp.de>
- http://home.schmorp.de/
+   Marc A. Lehmann <schmorp@schmorp.de>
+   http://software.schmorp.de/pkg/Coro.html
 
 =cut
 
